#include "tty.h"

/*#define DEBUG
 */

#define	BOFRAME 0xC0
#define	EOFRAME 0xC1
#define CESCAPE 0x7D

static u_char address = 0;
static u_char sendaddr[8] = { 0x00, 0x22, 0x44, 0x66, 0x88, 0xaa, 0xcc, 0xee };
static u_char recvaddr[8] = { 0x0e, 0x20, 0x42, 0x64, 0x86, 0xa8, 0xca, 0xec };
int totalpics = 0;

u_char
checksum(u_char addr, u_char *cp, int len)
{
    int ret = addr;
    while (len--)
	ret = ret + (*cp++);
    return  0x100-(ret & 0xff);
}

void 
sendcommand(u_char *p, int len)
{
    wbyte(BOFRAME);
    wbyte(sendaddr[address]);
    wstr(p, len);
    wbyte(checksum(sendaddr[address], p, len));
    wbyte(EOFRAME);
    address++; 
    address &= 7;
}

void
Abort()
{
    u_char buf[4];
    buf[0] = BOFRAME;
    buf[1] = 0x85;
    buf[2] = 0x7B;
    buf[3] = EOFRAME;
    wstr(buf, 4);
}  

int
recvdata(u_char *p, int len)
{
    u_char s, t;
    int sum;
    int i;

    s = rbyte();  /* BOFL */
    t= rbyte();  /* recvaddr */

    if (t != recvaddr[address]){
#ifdef DEBUG
	printf("not correct recv address\n");
#endif
	s = rbyte();  /* drain */
	s = rbyte();  /* drain */
	s = rbyte();  /* drain */
	Abort();
	return -1;
    }
    i = len;
    sum = (int) t;
    while ((s = rbyte()) != EOFRAME){
	sum = sum + s;
	if(i > 0){
	    if(s == CESCAPE){
		s = rbyte();
		if(0x20 & s)
		    s = 0xDF & s;
		else
		    s = 0x20 | s;
	    }
	    *p = s;
	    p++;
	    i--;
	}
	t = s;
    }
    if (sum & 0xff){
#ifdef DEBUG
	printf("check sum error.(%02x)\n", sum);
#endif
	return -1;
    }
    return len-i;
}

/*------------------------------------------------------------*/

int
CamReset()
{
    u_char buf[3];
    int retry = 5;

    while (retry--) {
	buf[0] = 0x01;
	buf[1] = 0x02;
	sendcommand(buf, 2);
	recvdata(buf, 3);
	if(buf[0] == 0x01 && buf[1] == 0x02 && buf[2] == 0x00)
	    return 0;
    }
    return -1;
}

int
CamOk()
{
    u_char buf[64];
    int retry = 5;

    memset(buf, 0, sizeof(buf));
    CamReset();
    while (retry--) {
	int i;
	buf[0] = 0x01;
	buf[1] = 0x01;
	sprintf(buf+2, "SONY     MKY-1001         1.00");
	sendcommand(buf, 32);
	if ((i = recvdata(buf, 34)) < 0) {
	    return -1;
	}
#ifdef DEBUG
	fprintf(stderr,"CamOk: len = %d, %d %d %d == 1 1 0\n", i, buf[0], buf[1], buf[2]);
	printf("00- %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c)\n", buf[0],  buf[0],  buf[1],  buf[1],  buf[2],  buf[2],  buf[3],  buf[3],  buf[4],  buf[4],  buf[5],  buf[5],  buf[6],  buf[6],  buf[7],  buf[7],  buf[8],  buf[8],  buf[9],  buf[9]);
	printf("10- %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c)\n", buf[10], buf[10], buf[11], buf[11], buf[12], buf[12], buf[13], buf[13], buf[14], buf[14], buf[15], buf[15], buf[16], buf[16], buf[17], buf[17], buf[18], buf[18], buf[19], buf[19]);
	printf("20- %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c)\n", buf[20], buf[20], buf[21], buf[21], buf[22], buf[22], buf[23], buf[23], buf[24], buf[24], buf[25], buf[25], buf[26], buf[26], buf[27], buf[27], buf[28], buf[28], buf[29], buf[29]);
	printf("30- %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c)\n", buf[30], buf[30], buf[31], buf[31], buf[32], buf[32], buf[33], buf[33], buf[34], buf[34], buf[35], buf[35], buf[36], buf[36], buf[37], buf[37], buf[38], buf[38], buf[39], buf[39]);
#endif
	if (buf[0] == 0x01 && buf[1] == 0x01 && buf[2] == 0x00)
	    return 0;
	else {
	    Abort();
	    CamReset();
	}
    }
    return -1;
}

int
CamStatus(int verbose)
{
    u_char buf[34];
    int pic_num2 = 0;
    int sw_mode = 0;
    int year, month, date;
    int hour, minutes;
    int i;

    memset(buf, 0, sizeof(buf));
    buf[0] = 0x03;
    buf[1] = 0x02;
    sendcommand(buf, 2);
    i = recvdata(buf, 34);
#ifdef DEBUG
    fprintf(stderr,"CamStatus: len = %d, %d %d %d == 3 2 0\n", i, buf[0], buf[1], buf[2]);
#endif
    if((buf[0] != 0x03) || (buf[1] != 0x02) ||(buf[2] != 0x00)){
	Abort();
	return -1;
    }

    sw_mode  = buf[3];
    totalpics  = (buf[4] << 8) + buf[5];
    pic_num2 = (buf[6] << 8) + buf[7];
    year     = (((buf[9] >> 4) * 10 + (buf[9] & 0x0f)) * 100) + (buf[10] >> 4) * 10 + (buf[10] & 0x0f);
    month    = (buf[11] >> 4) * 10 + (buf[11] & 0x0f);
    date     = (buf[12] >> 4) * 10 + (buf[12] & 0x0f);
    hour     = (buf[13] >> 4) * 10 + (buf[13] & 0x0f);
    minutes  = (buf[14] >> 4) * 10 + (buf[14] & 0x0f);
    
    if (verbose) {
	printf("Camera Mode: %s.\n%d pictures.\n%d:%02d %d/%d/%d\n\n", 
	       (sw_mode == 1 ? "play" : sw_mode == 2 ? "rec[a]" : "rec[m]"),
	       totalpics, hour, minutes, month, date, year);
    }
    
#ifdef DEBUG
    printf("00- %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c)\n", buf[0],  buf[0],  buf[1],  buf[1],  buf[2],  buf[2],  buf[3],  buf[3],  buf[4],  buf[4],  buf[5],  buf[5],  buf[6],  buf[6],  buf[7],  buf[7],  buf[8],  buf[8],  buf[9],  buf[9]);
    printf("10- %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c)\n", buf[10], buf[10], buf[11], buf[11], buf[12], buf[12], buf[13], buf[13], buf[14], buf[14], buf[15], buf[15], buf[16], buf[16], buf[17], buf[17], buf[18], buf[18], buf[19], buf[19]);
    printf("20- %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c)\n", buf[20], buf[20], buf[21], buf[21], buf[22], buf[22], buf[23], buf[23], buf[24], buf[24], buf[25], buf[25], buf[26], buf[26], buf[27], buf[27], buf[28], buf[28], buf[29], buf[29]);
    printf("30- %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c)\n", buf[30], buf[30], buf[31], buf[31], buf[32], buf[32], buf[33], buf[33], buf[34], buf[34], buf[35], buf[35], buf[36], buf[36], buf[37], buf[37], buf[38], buf[38], buf[39], buf[39]);
#endif
    return 0;
}

int
CamFopen(char *name)
{
    u_char buf[64];
    int i, len;

    memset(buf, 0, sizeof(buf));
    buf[0] = 0x02;
    buf[1] = 0x0A;
    buf[2] = 0x00;
    buf[3] = 0x00;
    sprintf(&buf[4], "%s", name);
    len = strlen(name) + 5;
    sendcommand(buf, len);
    i = recvdata(buf, 6);

#ifdef DEBUG
    fprintf(stderr,"CamFopen(%s): len = %d, %d %d %d == 2 10 0\n", name, i, buf[0], buf[1], buf[2]);
    printf("00- %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c) %02x(%c)\n", buf[0],  buf[0],  buf[1],  buf[1],  buf[2],  buf[2],  buf[3],  buf[3],  buf[4],  buf[4],  buf[5],  buf[5],  buf[6],  buf[6],  buf[7],  buf[7],  buf[8],  buf[8],  buf[9],  buf[9]);
#endif
    if ((buf[0] != 0x02) || (buf[1] != 0x0A) || (buf[2] != 0x00)) {
	Abort();
	return -1;
    }

    return buf[3];
}

int
CamFclose()
{
    u_char buf[4];
    int i;

    buf[0] = 0x02;
    buf[1] = 0x0B;
    buf[2] = 0x00;
    buf[3] = 0x00;
    sendcommand(buf, 4);
    i = recvdata(buf, 3);
 
#ifdef DEBUG
    fprintf(stderr,"CamFclose: len = %d, %d %d %d == 2 11 0\n", i, buf[0], buf[1], buf[2]);
#endif

    if (buf[0] != 0x02 || buf[1] != 0x0B || buf[2] != 0x00) {
	Abort();
	return -1;
    } else
	return 0;
}

long
CamFread(u_char *data, long len)
{
    long len2, i = 0;
    u_char s;

    u_char buf[10];

    buf[0] = 0x02;
    buf[1] = 0x0C;
    buf[2] = 0x00;
    buf[3] = 0x00;
    buf[4] = 0;
    buf[5] = 0;

    buf[6] = (len >> 8) & 0xff;
    buf[7] = 0xff & len;

    sendcommand(buf, 8);
    rstr(buf, 9);
    if (buf[2] != 0x02 || buf[3] != 0x0C || buf[4] != 0x00) {
	Abort();
	fprintf(stderr,"F1fread fail\n");
	return -1;
    }

    len2 = buf[7] * 0x100 + buf[8]; /* data size */
    if (len2 == 0) {
	s = rbyte(); /* last block checksum */
	s = rbyte(); /* last block EOFL */
	return 0;
    }  
    while ((s = rbyte()) != EOFRAME){
	if (s == CESCAPE){
	    s = rbyte();
	    if(0x20 & s)
		s = 0xDF & s;
	    else
		s = 0x20 | s;
	}
	if (i < len)
	    data[i] = s;
	i++;

    }
    i--; /* checksum */
    return i; 
}

u_long
CamFinfo(char *name)
{
    u_char buf[64];
    int len;
    u_long flen;

    buf[0] = 0x02;
    buf[1] = 0x0F;
    sprintf(&buf[2], "%s", name);
    len = strlen(name) + 3;

    sendcommand(buf, len);
    len = recvdata(buf, 37);
    if (buf[0] != 0x02 || buf[1] != 0x0F || buf[2] != 0x00) {
	Abort();
	return -1;
    }

    flen = buf[33] * 0x1000000 + buf[34] * 0x10000 + buf[35] * 0x100 + buf[36];

    return flen;
}

long
CamGetdata(char *name, u_char *data, int verbose)
{
    long filelen, total = 0, len;
    u_char *p;

    p = data;
    if ((filelen = CamFinfo(name)) < 0)
	return -1;

    if (CamFopen(name) != 0)
	return -1;

    while ((len = CamFread(p, 0x0400)) != 0){
	if (len < 0){
	    CamFclose();
	    return -1;
	}
	p = p + len;
	total = total + len;
    }
    CamFclose();
    return total;
}

int
CamFDelete(int n)
{
    u_char buf[4];
    int i;

    buf[0] = 0x02;
    buf[1] = 0x15;
    buf[2] = 0x00;
    buf[3] = 0xff & n;
    sendcommand(buf, 4);
    i = recvdata(buf, 3);
#ifdef DEBUG
    fprintf(stderr,"CamFDelete: len = %d, %d %d %d == 2 21 0\n", i, buf[0], buf[1], buf[2]);
#endif
    if (buf[0] != 0x02 || buf[1] != 0x15 || buf[2] != 0) {
	Abort();
	return -1;
    } else
	return 0;
}
